@@ -4,41 +4,37 @@ local package = pl.class(base)
 package._name = "inputfilter"
 
 function package:transformContent (content, transformFunction, extraArgs)
-  local newContent = {}
-  for k, v in SU.sortedpairs(content) do
-    if type(k) == "number" then
-      if type(v) == "string" then
-        local transformed = transformFunction(v, content, extraArgs)
-        if type(transformed) == "table" then
-          for i = 1, #transformed do newContent[#newContent+1] = transformed[i] end
-        else
-          newContent[#newContent+1] = transformed
-        end
+   local newContent = {}
+   for k, v in SU.sortedpairs(content) do
+      if type(k) == "number" then
+         if type(v) == "string" then
+            local transformed = transformFunction(v, content, extraArgs)
+            if type(transformed) == "table" then
+               for i = 1, #transformed do
+                  newContent[#newContent + 1] = transformed[i]
+               end
+            else
+               newContent[#newContent + 1] = transformed
+            end
+         else
+            newContent[#newContent + 1] = self:transformContent(v, transformFunction, extraArgs)
+         end
       else
-        newContent[#newContent+1] = self:transformContent(v, transformFunction, extraArgs)
+         newContent[k] = v
       end
-    else
-      newContent[k] = v
-    end
-  end
-  return newContent
+   end
+   return newContent
 end
 
 function package.createCommand (_, pos, col, lno, command, options, content)
-  local result = { content }
-  result.col = col
-  result.lno = lno
-  result.pos = pos
-  result.options = options
-  result.command = command
-  result.id = "command"
-  return result
+   local position = { lno = lno, col = col, pos = pos }
+   return SU.ast.createCommand(command, options, content, position)
 end
 
 function package:_init ()
-  base._init(self)
-  self:deprecatedExport("createCommand", self.createCommand)
-  self:deprecatedExport("transformContent", self.transformContent)
+   base._init(self)
+   self:deprecatedExport("createCommand", self.createCommand)
+   self:deprecatedExport("transformContent", self.transformContent)
 end
 
 package.documentation = [[
